class PcApp extends React.Component
{
  constructor(props)
  {
    super(props);

    setInterval(()=>{PcConfig.sayAlive()}, 1000);

    this.state = {initialized:false, device:props.device, renderSize:props.device=="surface" ? props.renderSize*0.75 : props.renderSize };
    this.app = React.createRef();
    this.mouseTimeout = null;
    this.createElementsAnimationStyle();

    PcConfig.initialize(async ()=> await this.onInitialized(), props.xml, props.host, props.lang); // FIXME: async constructor does not exist. defensive programming? how? https://dev.to/somedood/the-proper-way-to-write-async-constructors-in-javascript-1o8c
    PcConfig.renderSize = props.device=="surface" ? props.renderSize*0.75 : props.renderSize;
    PcConfig.device = props.device;
  }

  static getDerivedStateFromError(error)
  {
    if (PcConfig.useElectron())
    {
      let remote = require('electron').remote;
      let testMode = remote.getGlobal('sharedObj').testMode;

      if (testMode)
      {
        console.log("ERROR CATCHED IN TEST MODE");
      }
      else {
        console.log("ERROR CATCHED, RESTART IN 15 SECONDS");
        setTimeout(()=>{PcConfig.restartAll()}, 15000);
      }

    }
    else {
      console.log("ERROR CATCHED IN WEB MODE");
    }

    return {hasError:true};
  }

  async onInitialized()
  {
    await PcCaptureSession.initialize(()=>{
      this.setState({initialized:true});
    });
  }

  createElementsAnimationStyle()
  {
    var sheet = document.createElement('style');
    sheet.innerHTML = "@keyframes elementMoveDownAndFadeIn {from { opacity:0; transform:translateY(" + (-5*PcConfig.renderSize) + "px)} to { opacity:1;}}";
    document.body.appendChild(sheet);
  }

  onMouseMove()
  {
    clearTimeout(this.mouseTimeout);
    this.app.current.style.cursor="auto";

    this.mouseShowTimeout = setTimeout(()=>{
      this.app.current.style.cursor="none";
    }, 2000);
  }

  render()
  {
    if (!this.state.initialized)
    {
      return e("div",
        {
          ref:this.app,
          className:"pcapp",
          onMouseMove: ()=>this.onMouseMove(),
          style:{
            backgroundColor:"#000000",
            cursor:"none",
          }
        },
        e(PcScreenMainLoading,
          {
            active:true,
            renderSize:this.state.renderSize,
            width:document.body.clientWidth,
            height:document.body.clientHeight,
          }
        )
      );
    }
    else {

      let scenario;

      if (PcConfig.scenario.mode=="photobooth" || PcConfig.scenario.mode=="photokiosk" || PcConfig.scenario.mode=="cyclomaton")
      {
        scenario = e(PcScenario,
          {
            mode:PcConfig.scenario.mode,
            view:this.props.view,
            renderSize:this.state.renderSize,
            device:this.state.device
          }
        );
      }
      else if (PcConfig.scenario.mode=="videobooth" || PcConfig.scenario.mode=="videokiosk")
      {
        scenario = e(PcScenarioVideo,
          {
            mode:PcConfig.scenario.mode,
            view:this.props.view,
            renderSize:this.state.renderSize,
            device:this.state.device
          }
        );
      }

      return e("div",
        {
          ref:this.app,
          className:"pcapp",
          onMouseMove: ()=>this.onMouseMove(),
          style:{
            backgroundColor:"#000000",
            cursor:"none",
          }
        },
        scenario
      );
    }
  }

}
