var CONTROLDBG = false;
var INITDBG = true;
var CONSOLEDBG = false;
var FILEDBG = false;
class PcCameraControl {
  constructor() {
  }

  static async _waitFileExistsOrNot(file_or_dir, is_dir=false, check_delete=false, sleep_quanta_ms=100, sleep_time_ms=1000)
  {
    var fs = require('electron').remote.require("fs");
    var ret = false;
    var cond;
    var count = 0;
    var maxCount = sleep_time_ms / sleep_quanta_ms;
    var bad_cond_str = check_delete? "STILL " + (is_dir? "NOT EMPTY":"FOUND") : (is_dir? "EMPTY":"NOT FOUND");
    var good_cond_str = (check_delete? "NOT" : "") + " " + (is_dir? "EMPTY":" FOUND");
    var media_str =(is_dir? "DIRECTORY" : "FILE") + " " + file_or_dir;

    do {
      if (is_dir) {
        cond = (fs.readdirSync(file_or_dir).length != check_delete); // FIXME
      } else {
        cond = (fs.existsSync(file_or_dir) != check_delete);
      }
      if (cond) {
        ret = true; 
      } else { 
        count++;
        await this.sleep(sleep_quanta_ms);
      }
    } while (!cond && count <= maxCount)

    if (!ret) {
      PcConfig.dbg(true, "ERROR:", media_str, bad_cond_str, "AFTER", sleep_time_ms, "MS!");
    } else {
      PcConfig.dbg(FILEDBG, media_str, good_cond_str, "AS EXPECTED");
    }

    return ret;
  }

  static async waitFileExists(file, sleep_quanta_ms=100, sleep_time_ms=1000)
  {
    var ret = await this._waitFileExistsOrNot(file, false, false, sleep_quanta_ms, sleep_time_ms);
    return ret;
  }
  static async waitFileDeleted(file, sleep_quanta_ms=100, sleep_time_ms=1000)
  {
    var ret = await this._waitFileExistsOrNot(file, false, true, sleep_quanta_ms, sleep_time_ms);
    return ret;
  }

  static async waitDirEmpty(dir, sleep_quanta_ms=100, sleep_time_ms=1000)
  {
    var ret = await this._waitFileExistsOrNot(dir, true, true, sleep_quanta_ms, sleep_time_ms);
    return ret;
  }
  

  /* checks the file is synced and its size is > 0 */
  static async waitFileSync(file, sleep_quanta_ms=100, sleep_time_ms=1000)
  {
    var fs = require('electron').remote.require("fs");
    var synced = false;
    var size = 0;
    var maxCount = sleep_time_ms / sleep_quanta_ms;
    var count = 0;
    do {
      //PcConfig.dbg(true, "synced ? ", synced);
      try {
        fs.readFileSync(file, 'utf-8'); //did you know? this intricate loop could block restartAll before fix for bug #6
        synced = true;
      }
      catch (error) {
        //console.log (" ex" , error); // usually ebusy, not a problem, let's wait some more
        await this.sleep(sleep_quanta_ms);
      }
    } while (!synced && count <= maxCount);

    if (!synced) {
      PcConfig.dbg(true, "MEDIA", file, "NOT SYNCED AFTER", sleep_time_ms, "MS!");
    } else {
      PcConfig.dbg(FILEDBG, "MEDIA", file, "SYNCED AS EXPECTED");
    }

    // finally: check size (as the Recorder tool can provide 0-sized files without err code /stderr logs)

    size = fs.statSync(file).size;
    PcConfig.dbg(FILEDBG || size == 0, size > 0 ? "OK:":"ERROR:", "MEDIA", file, "SIZE", size);
    return synced && (size > 0);
  }
   
  static async checkFileCopiedOrDie(file, exists_sleep_quanta_ms=100, exists_sleep_time_ms=10000, sync_sleep_quanta_ms=100, sync_sleep_time_ms=10000)
  {
    var find = await this.waitFileExists(file, exists_sleep_quanta_ms, exists_sleep_time_ms);
    if (!find) {
      PcConfig.dbg(LEGACYDBG, "WAIT FILE EXISTS FAILED. RESTARTING");
      PcConfig.restartAll();
    }
    
   var synced = await this.waitFileSync(file, sync_sleep_quanta_ms, sync_sleep_time_ms);

   if (!synced){
    PcConfig.dbg(LEGACYDBG, "WAIT FILE SYNC FAILED. RESTARTING");
    PcConfig.restartAll();
   }
  }

  static useDSLR()
  {
    return !PcConfig.scenario.useWebcam;
  }
  static init(videoMode = false) {
    this.liveViewStarted = false;
    this.fileToRecord = "";
    this.webcamActive = false;
    this.dslrActive = false;

    if (PcConfig.useElectron()) { // electron application
      this.process = require('electron').remote.require("child_process");
      this.path = require('electron').remote.require("path");

    }

    // constants declaration, Display Aspect Ratios (DAR) for picture and video modes
    const PIC_DAR_NUM = 3;
    const PIC_DAR_DEN = 2;
    const VID_DAR_NUM = 16;
    const VID_DAR_DEN = 9;

    // electron DSLR resolutions:
    // picture mode
    const ELECTRON_DSLR_W = 640; // NOTE: when we use a DSLR under electron, whatever mode (picture, video) we cannot go beyond a width of 640 px 
    const ELECTRON_DSLR_H = ELECTRON_DSLR_W * PIC_DAR_DEN / PIC_DAR_NUM; // in picture, dar is 3/2 => ~= 426-ish
    // const ELECTRON_DSLR_H = "424"; // legacy value, to delete. note: dar = 1.509, a bit more than with 426 ^^^

    // video mode
    const ELECTRON_DSLR_W_REC = ELECTRON_DSLR_W;
    const ELECTRON_DSLR_H_REC = ELECTRON_DSLR_W_REC * VID_DAR_DEN / VID_DAR_NUM; // video mode: dar = 16/9 => 360px

    // electron webcam resolutions:
    // NOTE: We want a fixed height of 1080 px
    // picture mode
    const ELECTRON_WEBCAM_H = 1080;
    const ELECTRON_WEBCAM_W = ELECTRON_WEBCAM_H * PIC_DAR_NUM / PIC_DAR_DEN; // pic mode: dar = 3/2 => 1620

    // video mode 
    const ELECTRON_WEBCAM_H_REC = ELECTRON_WEBCAM_H;
    const ELECTRON_WEBCAM_W_REC = ELECTRON_WEBCAM_H_REC * VID_DAR_NUM / VID_DAR_DEN; // video mode: dar = 16/9 => 1920px

    // webview preview webcam resolutions:
    // NOTE: we want a fixed width of 1280px.... AND pic DAR = 16/9 instead of regular 3/2 (fixme later: IS IT BECAUSE SOME WEBCAMS ARE 16/9 ONLY?)
    // picture  mode
    const WEBVIEW_WEBCAM_W = 1280;
    const WEBVIEW_WEBCAM_H = WEBVIEW_WEBCAM_W * VID_DAR_DEN / VID_DAR_NUM ; // dodgy pic mode: dar = 16/9 (!) => 720px

    // video mode
    const WEBVIEW_WEBCAM_W_REC = WEBVIEW_WEBCAM_W; // in video mode with webview, video res = pic res
    const WEBVIEW_WEBCAM_H_REC = WEBVIEW_WEBCAM_H;

    let FINAL_W, FINAL_H, FINAL_W_REC, FINAL_H_REC, img_w, img_h, cam_w, cam_h, cam_w_rec, cam_h_rec;
    let arr;

    PcDbg.log(INITDBG, "PcCameraControl init(): useElectron? ", PcConfig.useElectron());
    PcDbg.log(INITDBG, "useWebcam? " + PcConfig.scenario.useWebcam);
    PcDbg.log(INITDBG, "useDSLR? ", this.useDSLR());

    if (PcConfig.useElectron()) { // final lengths against modes {electron,webview} x {dslr, webcam}
      if (this.useDSLR()) {
        arr = [ ELECTRON_DSLR_W, ELECTRON_DSLR_H, ELECTRON_DSLR_W_REC, ELECTRON_DSLR_H_REC ] ;

      } else if (PcConfig.scenario.useWebcam) {    
        arr = [ ELECTRON_WEBCAM_W, ELECTRON_WEBCAM_H, ELECTRON_WEBCAM_W_REC, ELECTRON_WEBCAM_H_REC ] ;
      }
    } else { // webview, no DSLR, so unique resoltution available  
      arr = [ WEBVIEW_WEBCAM_W, WEBVIEW_WEBCAM_H, WEBVIEW_WEBCAM_W_REC, WEBVIEW_WEBCAM_H_REC ] ;
    }

    // PcDbg.log(true, "arr: ", arr);
    // round our lengths and convert them to strings
    [ FINAL_W, FINAL_H, FINAL_W_REC, FINAL_H_REC ] = arr.map(i  => { return String(Math.floor(i)); } ); // round values, convert to string (legacy)

    [img_w, img_h] = [FINAL_W, FINAL_H];

    if (PcConfig.scenario.cameraOrientation == "portrait" || PcConfig.scenario.cameraOrientation == "portraitInverted") {
      [cam_w, cam_h, cam_w_rec, cam_h_rec] = [FINAL_H, FINAL_W, FINAL_H_REC, FINAL_W_REC];
    } else {
      [cam_w, cam_h, cam_w_rec, cam_h_rec] = [FINAL_W, FINAL_H, FINAL_W_REC, FINAL_H_REC];
    }
      
    PcDbg.log(INITDBG, "cam_h: ", cam_h, "cam_w: ", cam_w, "cam_h_rec: ", cam_h_rec, "cam_w_rec: ", cam_w_rec);

    this.cameraWidth = cam_w;
    this.cameraHeight = cam_h;
    this.cameraWidthWhileRecording = cam_w_rec;
    this.cameraHeightWhileRecording = cam_h_rec;
    this.imgWidth = img_w;
    this.imgHeight = img_h;
    this.img = new Image(this.imgWidth, this.imgHeight);


    this.liveViewTimeout = null;
    this.isShooting = false;

    this.video = document.createElement("video");

    this.videoMode = videoMode;
    this.videoRecording = false;

    this.canvas = document.createElement('canvas');
    this.canvas.width = this.cameraWidth;
    this.canvas.height = this.cameraHeight;

    this.context = this.canvas.getContext('2d');

    this.sleep = ms => new Promise(resolve => setTimeout(resolve, ms));

    this.startStamp = 0;
    this.flashStamp = [];


    this.initLiveView();
  }

  static initLiveView(fOnDo = null) {
    PcDbg.log(LEGACYDBG, "init liveview: useDSLR? " + this.useDSLR() + " because useWebCam? " + PcConfig.scenario.useWebcam);

    if(this.useDSLR()) {
    
      var cmd = "PATH=C:\\cygwin64\\bin;%PATH% && bash -c \"lsusb | egrep Nikon | head -1 | awk '{print $6}' | xargs echo -n \"" ; 
      // yes, head -1 ! sometimes lsusb sees dups, see bug #30 !
      var sout = PcConfig.myProcessExec(cmd, null, true);  //synchronous
        {
          /* //previous async code was:
          PcDbg.log(CONSOLEDBG, "error", error);
          PcDbg.log(CONSOLEDBG, "stdout", stdout, Buffer.from(stdout, 'utf8').toString('hex'));
          PcDbg.log(CONSOLEDBG, stderr);
          */
          switch(sout) 
          {
            case "04b0:0438": //NIKON D5500
            case "04b0:043f": //NIKON D5600
              this.dslrActive = true; 
              break;
            default:
                break;
          }
        }
    }

    if(! this.dslrActive) {
      return;
    } 
    var cmd = "\"" + PcConfig.global.recorderURL + "\" /Init";
    PcConfig.myProcessExec(cmd, (error, stdout, stderr) => // Recorder Init (liveView)
    {
      if (fOnDo) fOnDo();
    }
    );

  }
  
  static startLiveView() {
    PcDbg.log(LEGACYDBG, "enter startLiveView");
    
    if (this.liveViewStarted) {
      return;
    }
    
    PcDbg.log(LEGACYDBG, "DO startLiveView");
    
    if (this.dslrActive) { //from initLiveView
        var cmd = "\"" + PcConfig.global.recorderURL + "\" /StartLiveView";
        PcConfig.myProcessExec(cmd, () => // Recorder StartLiveView
        {
          this.liveViewStarted = true; //immediate feedback for getCurrentFrame()
          this.liveViewTimeout = setTimeout(() => PcCameraControl.img.src = "http://localhost:8083/?x=" + Math.random(), 500);
        } 
        );
    } else if (! this.useDSLR()  ) { /* do we want the webcam ? we need to probe it*/
        if (navigator.mediaDevices && navigator.mediaDevices.getUserMedia) {
              navigator.mediaDevices.getUserMedia({ video: { width: this.imgWidth, height: this.imgHeight } }).then((stream) => {
              this.video.srcObject = stream;
              this.video.play();
              this.webcamActive = true;
              this.liveViewStarted = true; //delayed feedback after getusermedia succeeds, for getCurrentFrame()
          }, () => {
              PcDbg.log(LEGACYDBG, "WEBCAM PROBING FAILURE!") 
              this.liveViewStarted = true; // delayed feedback after getusermedia fails,  for getCurrentFrame()
            }
          );
        }
        else { // no webcam available? 
          this.webcamActive = false;
          this.liveViewStarted = true; // probably immediate feedback after checking cameras and getusermedia exist immediate feedback for getCurrentFrame()
          PcDbg.log(true, "Can't find any webcam! ARGH");
        }
    } else { // no DSLR found yet DSLR was required
      PcDbg.log(true, "Can't find any DSLR! DAMN"); 
      this.liveViewStarted = true; // immediate feedback
    }
  }
  
  static stopLiveView() { 
    PcDbg.log(LEGACYDBG, "enter stopLiveView");

    if (!this.liveViewStarted) {
      return;
    }

    PcDbg.log(LEGACYDBG, "DO stopLiveView");

    this.liveViewStarted = false;
    if (this.dslrActive) {    
      var cmd = "\"" + PcConfig.global.recorderURL + "\" /StopLiveView";
      clearTimeout(this.liveViewTimeout);
      this.img.src = "";
      PcConfig.myProcessExec(cmd); // recorder StopLiveView
    
    } else if (this.webcamActive) {
      
      this.video.srcObject.getTracks().forEach(function(track) {
        if (track.readyState == 'live') {
            track.stop();
        }
      
      }
      );
    }
  }

  static autofocus() {
    if (this.dslrActive) {
      var cmd = "\"" + PcConfig.global.recorderURL + "\" /Autofocus";
      PcConfig.myProcessExec(cmd); // Recorder Autofocus
    }
  }

  static async shoot() {
    //PcCameraControl.stopLiveView();
    if (this.dslrActive == false) {
      await Promise.all([PcConfig.promiseSleep(1750)]);
    }
    await this.doShoot();

    //setTimeout(()=>PcCameraControl.startLiveView(), 2000);
  }

  static startVideoRecording() {
    if (this.videoRecording) return;

    if (this.dslrActive) {
      var cmd = "\"" + PcConfig.global.recorderURL + "\" /StartRecording";
      PcConfig.myProcessExec(cmd, () => { // Recorder StartRecording video
        this.videoRecording = true;
        PcDbg.log(LEGACYDBG, "recording video " + Date.now() / 1000);
        while (this.flashStamp.length) { this.flashStamp.pop(); }
        this.startStamp = Date.now() / 1000;
      });
    }
  }

  static stopVideoRecording() {
    if (this.dslrActive) {
      var cmd = "\"" + PcConfig.global.recorderURL + "\" /StopRecording";
      PcConfig.myProcessExec(cmd, () => this.videoRecording = false); // Recorder StopRecording video
    }
  }

  static async transferVideoRecording(fOnTransfer) {
    if (this.dslrActive) {
      let filename = (new Date().getTime() + Math.floor((Math.random() * 10000) + 1)).toString(16) + ".mov";
      let fileToRecord = PcConfig.global.recordedFolder + filename;

      var cmd = "\"" + PcConfig.global.recorderURL + "\" /Transfer " + fileToRecord;

      var files_to_del = "\"" + PcConfig.global.recordedFolderForDelete + "\\*\"";
      PcConfig.myProcessExec("del /q " + files_to_del, null, true); // synchronous
      {
      //PcConfig.dbg(true, "err: ", err, "stdout:", stdout, "stderr:", stderr);
        
        var deleted = await this.waitDirEmpty(PcConfig.global.recordedFolderForDelete);
        if (!deleted) {
          PcConfig.dbg(LEGACYDBG, "WAIT DIR EMPTY", PcConfig.global.recordedFolderForDelete, "EMPTY FAILED. RESTARTING");
          PcConfig.restartAll();     
        }
       }; // Delete recorderFolderForDelete (del /q ...) synchronously

      PcConfig.myProcessExec(cmd, null, true); //// Recorder Transfer Video Recording Synchronously)
      { 
        /*
          ALERT: the Recorder tool silently fails when the camera is disconnected. 
          one can see a stacktrace on stdout, but nothing on stderr and err code is null (!!)
          so the only remaining criteria is to check if the returned file is NOT empty
        */
        //PcConfig.dbg(true, "err: ", err, "stdout:", stdout, "stderr:", stderr);
        await this.checkFileCopiedOrDie(fileToRecord, 1000, 30000 /* cooler sync sleeps than default */);
        PcConfig.setMemory('recordedVideoFile', fileToRecord);
        PcConfig.setMemory('recordedVideoFileName', filename);
        if (fOnTransfer) fOnTransfer();
      }; // also synchronously
    }
    else {
      PcConfig.setMemory('recordedVideoFile', PcConfig.host + 'ressources/videodemo.mp4');
      if (fOnTransfer) fOnTransfer();
    }
  }

  static async doBoommerangShoot() {
    PcDbg.log(CRASHDBG, "enter doBoomerangShoot");
    let canvas = document.createElement('canvas');
    canvas.width = this.video.videoWidth;
    canvas.height = this.video.videoHeight;
    canvas.getContext('2d').drawImage(this.video, 0, 0, this.video.videoWidth, this.video.videoHeight);
    await PcCaptureSession.addFrame(canvas, true);
    PcDbg.log(CRASHDBG, "exit doBoomerangShoot");
  }

  static async doShoot() {
    PcDbg.log(LEGACYDBG, "enter DO SHOOT");
    let fs = require('electron').remote.require("fs");
    if(this.dslrActive) {
      var n = PcCaptureSession.shots.length;
      var uniqid = PcConfig.getUniqId();
      var name = "photo" + "_" + uniqid + "_" + n + ".jpg";
      var file = PcConfig.global.recordedFolder + name;
      var hostedFile = PcConfig.host + PcConfig.global.recordedFolderForSave + name;
      var cmd = "\"" + PcConfig.global.recorderURL + "\" /c " + file;

      PcConfig.myProcessExec(cmd, async () => { // Recorder /c (shoot photo)
        await this.checkFileCopiedOrDie(file);
        await PcCaptureSession.addFrame(file);
        PcCaptureSession.nextShot(); 
      });

    }
    else if(this.webcamActive) // webcam active (electron with useWebcam = true, or web view where webcam was detected)
    {
      let canvas = document.createElement('canvas');
      canvas.width = this.video.videoWidth;
      canvas.height = this.video.videoHeight;
      canvas.getContext('2d').drawImage(this.video, 0, 0, this.video.videoWidth, this.video.videoHeight);
      await PcCaptureSession.addFrame(canvas, true);
      PcCaptureSession.nextShot();
    }
    else //webcam doesn't exist (electron with useWebcam = false, or web view where webcam was not detected)
    {
      PcDbg.log(LEGACYDBG, "doShoot: NO DSLR, NO WEBCAM");
      let canvas = document.createElement('canvas');
      canvas.width = 800;
      canvas.height = 450;
      canvas.fillStyle = "black";
      canvas.getContext('2d').fillRect(0, 0, canvas.width, canvas.height);
      await PcCaptureSession.addFrame(canvas, true);
      PcCaptureSession.nextShot();
    }

    PcDbg.log(CRASHDBG, "exit DO SHOOT");

  }

  static exitDSLR() {
    if (this.dslrActive) {
      var cmd = "\"" + PcConfig.global.recorderURL + "\" /Exit";
      PcConfig.myProcessExec(cmd); // Recorder Exit
      this.dslrActive = false;
    }
  }

  static exitDSLRStrong(fOnDo = null) {
    if (this.dslrActive) {
      let cmd = "";

      this.stopVideoRecording();
      this.stopLiveView();

      cmd = 'Taskkill /IM Photocabine.Process.exe /F /T';
      setTimeout(() => {
        //console.log("taskkill Photocabine.process.exe");
        PcConfig.myProcessExec(cmd, () => { // Taskkill PhotocabineProcess.exe 
          if (fOnDo) fOnDo();
          this.dslrActive = false;
        });
      }, 1000);
    }
  }
  static myFillText(context, txt, x, y) {

    // get a pseudo height for the font
    var fontParts = context.font.split(' ');
    var fontHeight = parseInt(fontParts.length === 2 ? fontParts[0] : fontParts[1], 10);
    // split the text based on HTML5 line-breaks
    //var txtParts = txt.toString().replaceAll('<br>', '$').split('$'); //replaceAll('<br>', '$').replaceAll('<br/>', '$').split('$'); // unsupported in electron
    var delims_res = [new RegExp("<br/?>", 'g')];
    let txt_repl = txt.toString();
    PcDbg.log(false, "delims_res: " +delims_res);
    delims_res.forEach(re => {
      PcDbg.log(false, "re: " + re);
      txt_repl = txt_repl.replace(re, '$');
    });

    var txtParts = txt_repl.split('$');
    //note: replaceAll() doesn't work here

    // iteate parts and increase line position for each line
    for(var i=0; i < txtParts.length; i++) {
        context.fillText(txtParts[i], x, y + i * fontHeight);
    }
}
  static getCurrentFrame(destW, destH) {
    var cameraWidth, cameraHeight, cameraX, cameraY;
    var destCW, destCH, destX, destY;
    var oW, oH, oX, oY;

    if (PcConfig.getMemory("currentFormat").animation == "boomerang") {
      cameraHeight = this.cameraHeightWhileRecording;
    }
    else {
      if (this.img.naturalHeight == this.cameraHeightWhileRecording) {
        cameraHeight = this.cameraHeightWhileRecording;
      }
      else {
        cameraHeight = this.cameraHeight;
      }
    }

    if (PcConfig.scenario.mode == "photobooth" && !PcConfig.screen.format.wideCapture && !PcConfig.getMemory("currentFormat").autocrop) {
      cameraWidth = Math.min(this.cameraWidth, cameraHeight);
      cameraHeight = cameraWidth;
      cameraX = (this.cameraWidth - cameraWidth) / 2;

      if (this.img.naturalHeight == this.cameraHeightWhileRecording) {
        cameraY = 0;
      }
      else {
        cameraY = (this.cameraHeight - cameraHeight) / 2;
      }
    }
    else if (PcConfig.scenario.mode == "videobooth" || PcConfig.scenario.mode == "videokiosk") {
      cameraWidth = this.cameraWidth;
      cameraX = 0;
      cameraY = 0;

      if (this.img.naturalHeight == this.cameraHeightWhileRecording) {
        cameraHeight = this.cameraHeightWhileRecording;
      }
      else {
        cameraHeight = this.cameraHeight;
      }
    }
    else {
      cameraWidth = this.cameraWidth;
      cameraX = 0;
      cameraY = 0;
    }

    if ((destW / destH) < (cameraWidth / cameraHeight)) {
      oH = cameraHeight;
      oW = cameraHeight * destW / destH;
      oY = cameraY;
      oX = cameraX + (cameraWidth - oW) / 2;
    }
    else {
      oW = cameraWidth;
      oH = cameraWidth * destH / destW;
      oY = cameraY + (cameraHeight - oH) / 2;
      oX = cameraX;
    }

    let canvas = document.createElement('canvas');
    canvas.width = destW;
    canvas.height = destH;
    let context = canvas.getContext('2d');
    let device = this.useDSLR() ? "Appareil photo" : "Webcam"; // fixme later: internationalization / back-end administrable text
    try {
      PcDbg.log(CONTROLDBG, "getCurrentFrame: dslrActive? " + this.dslrActive + " webcamActive? " + this.webcamActive);
      if (this.dslrActive || this.webcamActive) {
        let source;
        if (this.dslrActive) {
          source = this.img;
        } else if (this.webcamActive) {
          source = this.video;
        }
        if (PcConfig.scenario.cameraOrientation == "portrait" || PcConfig.scenario.cameraOrientation == "portraitInverted") {
          let target;
          let ndw;
          let ndh;

          let newcanvas = document.createElement('canvas');
          let newcontext = newcanvas.getContext('2d');

          if (this.dslrActive) {
            PcDbg.log(CONTROLDBG, "dslrActive!");
            target = PcConfig.cloneCanvas(source);            
                        
          } else if (this.webcamActive) {
            PcDbg.log(CONTROLDBG, "webcamActive!");
            let videoCanvas = document.createElement('canvas');
            let videoContext = videoCanvas.getContext('2d');
            videoCanvas.width = this.cameraHeight;
            videoCanvas.height = this.cameraWidth;
            target = videoCanvas;

            PcDbg.log(CONTROLDBG, "Portrait Webcam DI1");
            videoContext.drawImage(source, 0, 0);
          }
          ndw = target.width;
          ndh = target.height;

          newcontext.clearRect(0, 0, newcanvas.width, newcanvas.height);
          newcanvas.width = ndh;
          newcanvas.height = ndw;
          newcontext.save();
          newcontext.translate(newcanvas.width / 2, newcanvas.height / 2);

          if (PcConfig.scenario.cameraOrientation == "portrait") {
            newcontext.rotate(-90 * Math.PI / 180);
          } else { // portrait inverted
            newcontext.rotate(90 * Math.PI / 180);
          }
          PcDbg.log(CONTROLDBG, "Portrait Common DI2");
          newcontext.drawImage(target, -ndw / 2, -ndh / 2, ndw, ndh);
          newcontext.restore();
          PcDbg.log(CONTROLDBG, "Portrait Common DI3");
          context.drawImage(newcanvas, oX, oY, oW, oH, 0, 0, destW, destH);
        } else {  // landscape  
          PcDbg.log(CONTROLDBG, "landscape");
          context.drawImage(source, oX, oY, oW, oH, 0, 0, destW, destH);
        }
      } else if (this.liveViewStarted) {  // neither dsl or webcam active, yet initLiveView was completed
        context.fillStyle = "#333333";
        context.fillRect(0, 0, canvas.width, canvas.height);
        context.fillStyle = "#888888";
        context.scale(-1, 1);
        context.font = "32px Arial";
        context.textAlign = "center";
        let txt;
        if (PcConfig.screen.start.textNoDeviceDetected && PcConfig.screen.start.textNoDeviceDetected.length > 0) {
          txt = PcConfig.screen.start.textNoDeviceDetected;
          
        } else {
          txt = "absent(e)";
        }
        this.myFillText(context,device + " " + txt, -canvas.width / 2, canvas.height / 2); // fixme later: make text and font size and color admnistrable in back-end
      }
    }
    catch (error) {
      console.log("ERROR getCurrentFrame failed with EXCEPTION:" + error);
      PcCameraControl.img.src = "http://localhost:8083/?x=" + Math.random();
    }

    context = null;

    return canvas;
  }



  static deleteOldShoots() {
    if (!this.dslrActive) return;

    const directory = PcConfig.global.recordedFolder;

    let fs = require('electron').remote.require("fs");

    fs.readdir(directory, (err, files) => {
      if (err) throw err;

      let path = require('electron').remote.require("path");

      for (const file of files) {
        fs.unlink(path.join(directory, file), err => {
          if (err) throw err;
        });
      }
    });
  }

}
